import app.landscape

#
# And HAProxy cluster comprises 1 or more HAProxy machines that serve as load balancer.
# HAProxy can handle multiple services -- where each service is either backed
# by a cluster (real load balacing) or a single machine (reverse proxying).
#
# +----------------+               +-----------------+    +----------------+
# | HAProxy cluster| --(ctfcore)-->| CTF Core cluster| -->| CTFCore machine|-+
# +----------------+ \             +-----------------+    +----------------+ |
#                     |                                     +----------------+
#                     | 
#                     |            +-----------------+    +----------------+
#                     +-----(svn)->|   SVN cluster   | -->|   SVN machine  |-+
#                     |            +-----------------+    +----------------+ |
#                     |                                     +----------------+
#                     | 
#                     |                                   +----------------+
#                     +-----(svn)------------------------>|   SVN machine  |
#                                                         +----------------+ 
#
class HaproxyCluster(object):
    def __init__(self, cluster):
        self.__cluster = cluster

    @staticmethod
    def get():
        localhost = app.landscape.Landscape.getSite().getLocalMachine()
        if not localhost.isClustered():
            raise Exception("Encountered invalid configuration - the local machine is not part of a cluster")
            
        return HaproxyCluster(localhost.getCluster())
        
    def __getBackendForService(self, serviceName):
        backendName = app.landscape.Landscape.getSiteOptions().get("%s:%s:BACKEND" % (
            self.__cluster.getName(), 
            serviceName))
        if not backendName:
            raise Exception("No backend defined for HAProxy cluster '%s' and service '%s'" % 
                (self.__cluster.getName(), serviceName))
                
        return backendName
        
    def __getServices(self):
        clusterName = self.__cluster.getName()
    
        return [key.split(":")[1] 
            for key in app.landscape.Landscape.getSiteOptions().keys()
            if key.startswith(clusterName + ":") and key.endswith(":BACKEND")]
    
    def getBackendFqdns(self, serviceName, endpointName):
        flatten = lambda l: [item for sublist in l for item in sublist]
        
        # Of machines that we are load balancing for...
        machines = flatten([self.getBackendMachines(service) for service in self.__getServices()])
        
        # get all endpoints that are exposed via <serviceName/endpointName>...
        viaTargets = flatten([m.getService(serviceName).getProvidedEndpoint(endpointName).getViaTargetEndpoints() 
            for m in machines
            if m.hasService(serviceName)])
        
        # and then get the set of FQDNs they use.
        return set([v.getHost(False).lower() for v in viaTargets])
        
    def getBackendMachines(self, serviceName):
        site = app.landscape.Landscape.getSite()
        backend = self.__getBackendForService(serviceName)
        
        backendCluster = site.getCluster(backend)
        if backendCluster:
            # Clustered backend.
            return backendCluster.getMachines()
            
        backendMachine = site.getMachine(backend)
        if backendMachine:
            # Single machine as backend.
            return [backendMachine]
            
        raise Exception("Backend '%s' for service '%s' not defined" % (backend, serviceName))